// Hanoi.java

import java.awt.*;
import javax.swing.*;
import ch.aplu.util.*;
import java.awt.event.*;
import java.util.Stack;

public class Hanoi extends JApplet implements Runnable,
    ExitListener
{
// ------------------ Inner class Disk --------------------
  private class Disk
  {
    private int size;
    private Color color;

    public Disk(int diskSize)
    {
      size = diskSize;
      color = new Color((1 * size) % 256, (40 * size) % 256,
                        (40 * size) % 256);
    }
  }

// ----------- Inner class MyButtonListener ---------------
  private class MyButtonListener implements ActionListener
  {
    public void actionPerformed(ActionEvent evt)
    {
      if (play) // Inhibit button when playing
        return;
      nbDisks = Integer.parseInt(textField.getText());
      for (int i = 0; i < 3; i++)
        towers[i] = new Stack();
      for (int i = nbDisks; i > 0; i--)
        towers[0].push(new Disk(i));
      p.window(0, 6 * nbDisks, 0, 3 * nbDisks);
      play = true;
    }
  }

// ------------- Instance variables -----------------------
  private GPanel p;
  private int nbDisks;
  private int nbMoves;
  private Stack[] towers = new Stack[3];
  private JTextField textField = new JTextField(3);
  private JLabel label = new JLabel("Number of disks");
  private JButton goButton = new JButton("Go");
  private boolean play;
  private volatile boolean active;

  public void start()
  {
    p = new GPanel(GPanel.APPLETFRAME);
    p.addExitListener(this);
    p.title("Towers of Hanoi");
    p.bgColor(Color.white);
    p.add(label);
    p.add(textField);
    p.add(goButton);
    goButton.addActionListener(new MyButtonListener());
    p.validate();
    textField.setText("5"); // Default
    textField.requestFocus();
    nbMoves = 0;
    play = false;
    new Thread(this).start();
  }

  public void stop()
  {
    active = false;
    p.dispose();
  }

  public void run()
  {
    active = true;
    while (active)
    {
      if (play)
      {
        draw();
        go(towers[0], towers[1], towers[2], nbDisks);
        nbMoves = 0;
        play = false;
      }
    }
  }

  public void notifyExit()
  {
    active = false;
    p.dispose();
  }

  private void draw()
  {
    p.clear();
    p.color(Color.black);
    p.move(0, 0);
    p.text("Number of moves: " + nbMoves);
    for (int i = 0; i < 3; i++)
    {
      p.color(Color.black);
      p.move(nbDisks * (2 * i + 1), 1.5 * nbDisks);
      p.fillRectangle(0.2, nbDisks + 1);
    }
    for (int i = 0; i < 3; i++)
    {
      for (int k = towers[i].size() - 1; k >= 0; k--)
      {
        Disk current = (Disk)towers[i].elementAt(k);
        p.color(current.color);
        p.move((2 * i + 1) * nbDisks, nbDisks + k);
        p.fillRectangle(2 * current.size, 0.9);
      }
    }
  }

  private void go(Stack source, Stack help, Stack target,
                  int n)
  {
    if (!active)
      return;

    if (n == 1)
    {
      nbMoves++;
      target.push(source.pop());
      Console.delay(1000);
      draw();
    }
    else
    {
      go(source, target, help, n - 1);
      go(source, towers[0], target, 1);
      go(help, source, target, n - 1);
    }
  }

  public static void main(String[] args)
  {
    new Hanoi().start();
  }
}
